﻿/*
 * TismTool © Tis Veugen, 2010
 * 
 * TsmComponent.cs
 * version 1
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TsmSupervisor;

namespace TsmCode
{
  public class TsmClass : TsmBase
  {
    public const int  NativePortId = -1;
    public TsmFsm []  oFsms;
    public bool       IsMock = false;
    public string     SnapState = null;
    public string     SnapCall = "";

    public virtual void SetProxyUser( TsmProxyUser oProxyUser )
    {
      // ignore
    }

    public virtual TsmInterfacePort GetProvidedInterface( string sPortName )
    {
      throw new Exception(
        string.Format( "9500: GetProvidedInterface should be overriden for '{0}' @ '{1}", sPortName, this ) );
    }

    public virtual void SetRequiredInterface( string sPortName, TsmInterfacePort oProvidedInterface )
    {
      throw new Exception(
        string.Format( "9510: SetRequiredInterface should be overriden for '{0}' @ '{1}", sPortName, this ) );
    }

    public virtual void ExecuteTrigger( TsmTrigger oTrigger )
    {
    }

    public void ExecuteNative( int iCmd )
    {
      TsmTrigger lTrigger = new TsmTrigger( iCmd );
      lTrigger.PortId = NativePortId;
      ExecuteTrigger( lTrigger );
    }

    public virtual void TsmInitialise( object oObject )
    {
    }

    public virtual void TsmConnect( object oObject )
    {
    }

    public virtual void TsmPrepare( object oObject )
    {
    }

    public virtual void TsmRun( object oObject )
    {
    }

    public virtual void TsmHalt( object oObject )
    {
    }

    public virtual void TsmCease( object oObject )
    {
    }

    public virtual void TsmDisconnect( object oObject )
    {
    }

    public virtual void TsmTerminate( object oObject )
    {
    }

    public virtual void TsmMention( object oObject )
    {
      List<TsmClass> ClassList = (List<TsmClass>) oObject;

      ClassList.Add( this );
    }

    public void TsmReset()
    {
      for ( int i=0; i<oFsms.Count(); i++ )
      {
        oFsms[i].TsmReset( new TsmHandler( this ) );
      }
      SnapState = null; // Invalidate SnapState
    }

    public string TsmSnap()
    {
      if ( null == SnapState )
      {
        StringBuilder sb = new StringBuilder();
        sb.Append( "{" + this.TsmName );
        for ( int i=0; i<oFsms.Count(); i++ )
        {
          sb.Append( "/" + oFsms[i].TsmSnap() );
        }
        SnapState = sb.ToString() + "}";
      }
      return SnapState + SnapCall;
    }

    public virtual void TsmCheckVisit()
    {
      for ( int i=0; i<oFsms.Count(); i++ )
      {
        oFsms[i].TsmCheckVisit();
      }
    }
  }

  public class TsmProtocolClass : TsmClass
  {
  }

  /**
   * Component
   */
  public class TsmComponent : TsmClass
  {
    public int             mNrClasses = 0;
    public TsmClass []     mClasses = null;
    public int             mNrComponents = 0;
    public TsmComponent [] mComponents = null;

    protected void DoAssembly( TsmClass Client, string sRequired, TsmClass Server, string sProvided )
    {
      TsmInterfacePort lPort;

      lPort = Server.GetProvidedInterface( sProvided );
      Client.SetRequiredInterface( sRequired, lPort );
    }

    public virtual void TsmInitialiseAll( object oObject )
    {
      int i;

      for ( i = 0; i < mNrClasses; i++ )
      {
        mClasses[i].TsmInitialise( oObject );
      }
      for ( i = 0; i < mNrComponents; i++ )
      {
        mComponents[i].TsmInitialiseAll( oObject );
      }

      this.TsmInitialise( oObject );
    }

    public virtual void TsmConnectAll( object oObject )
    {
      int i;

      for ( i = 0; i < mNrClasses; i++ )
      {
        mClasses[i].TsmConnect( oObject );
      }
      for ( i = 0; i < mNrComponents; i++ )
      {
        mComponents[i].TsmConnectAll( oObject );
      }

      this.TsmConnect( oObject );
    }

    public virtual void TsmPrepareAll( object oObject )
    {
      int i;

      for ( i = 0; i < mNrClasses; i++ )
      {
        mClasses[i].TsmPrepare( oObject );
      }
      for ( i = 0; i < mNrComponents; i++ )
      {
        mComponents[i].TsmPrepareAll( oObject );
      }

      this.TsmPrepare( oObject );
    }

    public virtual void TsmRunAll( object oObject )
    {
      int i;

      for ( i = 0; i < mNrClasses; i++ )
      {
        mClasses[i].TsmRun( oObject );
      }
      for ( i = 0; i < mNrComponents; i++ )
      {
        mComponents[i].TsmRunAll( oObject );
      }

      this.TsmRun( oObject );
    }

    public virtual void TsmHaltAll( object oObject )
    {
      int i;

      for ( i = 0; i < mNrClasses; i++ )
      {
        mClasses[i].TsmHalt( oObject );
      }
      for ( i = 0; i < mNrComponents; i++ )
      {
        mComponents[i].TsmHaltAll( oObject );
      }

      this.TsmHalt( oObject );
    }

    public virtual void TsmCeaseAll( object oObject )
    {
      int i;

      for ( i = 0; i < mNrClasses; i++ )
      {
        mClasses[i].TsmCease( oObject );
      }
      for ( i = 0; i < mNrComponents; i++ )
      {
        mComponents[i].TsmCeaseAll( oObject );
      }

      this.TsmCease( oObject );
    }

    public virtual void TsmDisconnectAll( object oObject )
    {
      int i;

      for ( i = 0; i < mNrClasses; i++ )
      {
        mClasses[i].TsmDisconnect( oObject );
      }
      for ( i = 0; i < mNrComponents; i++ )
      {
        mComponents[i].TsmDisconnectAll( oObject );
      }

      this.TsmDisconnect( oObject );
    }

    public virtual void TsmTerminateAll( object oObject )
    {
      int i;

      for ( i = 0; i < mNrClasses; i++ )
      {
        mClasses[i].TsmTerminate( oObject );
      }
      for ( i = 0; i < mNrComponents; i++ )
      {
        mComponents[i].TsmTerminateAll( oObject );
      }

      this.TsmTerminate( oObject );
    }

    public virtual void TsmMentionAll( object oObject )
    {
      int i;

      for ( i = 0; i < mNrClasses; i++ )
      {
        mClasses[i].TsmMention( oObject );
      }
      for ( i = 0; i < mNrComponents; i++ )
      {
        mComponents[i].TsmMentionAll( oObject );
      }
    }

    public virtual void TsmCheckVisitAll()
    {
      int i;

      for ( i = 0; i < mNrClasses; i++ )
      {
        mClasses[i].TsmCheckVisit();
      }
      for ( i = 0; i < mNrComponents; i++ )
      {
        mComponents[i].TsmCheckVisitAll();
      }
    }
  }

  /**
   * Executable Component
   */
  public class TsmExecComponent : TsmComponent
  {
    public TsmExecComponent()
    {
      // InitializeComponent();
    }

    public void Activate()
    {
      TsmSupervisor.TsmSupervisor.Initialise();
      TsmIntQueue lIntQueue = TsmSupervisor.TsmSupervisor.GetIntQueue();

      TsmInitialiseAll( null );
      TsmConnectAll( lIntQueue );
      TsmPrepareAll( null );
      TsmRunAll( null );
    }

    public void Execute()
    {
      TsmSupervisor.TsmSupervisor.Execute();
    }

    public void Deactivate()
    {
      TsmHaltAll( null );
      TsmCeaseAll( null );
      TsmDisconnectAll( null );
      TsmTerminateAll( null );

      TsmSupervisor.TsmSupervisor.Terminate();
    }
  }

  /**
   * Passive Component
   */
  public class TsmPComponent : TsmComponent
  {
    public override void TsmPrepare( object oObject )
    {
    }
  }

  /**
   * Active Component
   */
  public class TsmAComponent : TsmComponent
  {
    private TsmThread mThread;

    public TsmAComponent()
    {
      mThread = new TsmThread();
    }

    public TsmThread Thread
    {
      get { return mThread; }
      set { mThread = value; }
    }

    public override void TsmConnectAll( object oObject )
    {
      base.TsmConnectAll( Thread.IntQueue );
    }

    public override void TsmPrepare( object oObject )
    {
      mThread.Prepare();
    }

    public override void TsmRun( object oObject )
    {
      mThread.Run();
    }

    public override void TsmHalt( object oObject )
    {
      mThread.Halt();
    }

    public override void TsmCease( object oObject )
    {
      mThread.Cease();
    }
  }

  public class TsmProxy : TsmClass
  {
    private TsmProxyUser mUser;

    public override void SetProxyUser( TsmProxyUser oProxyUser )
    {
      mUser = oProxyUser;
    }

    public void SetProxyOk()
    {
      mUser.CbSetProxyOk();
    }

    public void SetProxyNok()
    {
      mUser.CbSetProxyNok();
    }
  }


  public class TsmProxyUser : TsmClass
  {
    private bool mProxyOk;

    public bool IsProxyOk()
    {
      return mProxyOk;
    }

    public bool IsProxyNok()
    {
      return ! mProxyOk;
    }

    public void CbSetProxyOk()
    {
      mProxyOk = true;
    }

    public void CbSetProxyNok()
    {
      mProxyOk = false;
    }
  }

  public class TsmInterfacePort : TsmBase
  {
    private int       mPortId;
    private TsmPort   mMessager;
    private TsmClass  mSource;

    public int PortId
    {
      get { return mPortId; }
      set { mPortId = value; }
    }

    public TsmPort Messager
    {
      get { return mMessager; }
      set { mMessager = value; }
    }

    public TsmClass Source
    {
      get { return mSource; }
      set { mSource = value; }
    }

    public void SetProxyUser( TsmProxyUser oProxyUser )
    {
      mMessager.SetProxyUser( oProxyUser );
    }
  }

  public class TsmPort
  {
    private TsmClass mTarget;

    public TsmClass Target
    {
      get { return mTarget; }
      set { mTarget = value; }
    }

    public void SetProxyUser( TsmProxyUser oProxyUser )
    {
      mTarget.SetProxyUser( oProxyUser );
    }

    public virtual void Handle( TsmTrigger oTrigger )
    {
    }
  }

  /**
   * Passive Server Port
   */
  public class TsmPSPort : TsmPort
  {
    public override void Handle( TsmTrigger oTrigger )
    {
      Target.ExecuteTrigger( oTrigger );
    }
  }

  /**
   * Passive Client Port
   */
  public class TsmPCPort : TsmPort
  {
    private TsmIntQueue mIntQueue;

    public TsmIntQueue IntQueue
    {
      set { mIntQueue = value; }
    }

    public override void Handle( TsmTrigger oTrigger )
    {
      TsmEnvelope lEnvelope = new TsmEnvelope( oTrigger, Target );
      mIntQueue.Put( lEnvelope );
    }
  }

  public class TsmActivePort : TsmPort
  {
    protected TsmThread mThread;

    public TsmThread Thread
    {
      set { mThread = value; }
    }
  }

  /**
   * Active Server Port
   * N.B. The PortId should be the PortId of the class' passive port !
   */
  public class TsmASPort : TsmActivePort
  {
    public override void Handle( TsmTrigger oTrigger )
    {
      TsmEnvelope lEnvelope = new TsmEnvelope( oTrigger, Target );
      mThread.SyncCall( lEnvelope );
    }
  }

  /**
   * Active Client Port
   * N.B. The PortId should be the PortId of the class' passive port !
   */
  public class TsmACPort : TsmActivePort
  {
    public override void Handle( TsmTrigger oTrigger )
    {
      TsmEnvelope lEnvelope = new TsmEnvelope( oTrigger, Target );
      mThread.ASyncCall( lEnvelope );
    }
  }
}
