﻿/*
 * TismTool © Tis Veugen, 2010
 * Application
 *
 * TsmGen.cs
 * version 4
 */
using System;
using System.IO;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Runtime.Remoting.Contexts;
using TsmSupervisor;

namespace TsmCode
{
  public class TsmBase
  {
    public TsmBase TsmParent;
    public string TsmName;

    public virtual string TotalName()
    {
      string s = "";
      if ( null != TsmParent )
      {
        s = TsmParent.TotalName();
        string p = s.Substring( s.Length - 1, 1 );
        if ( p != ">" )
        {
          s += " ";
        }
        s += ">";
        if ( ! string.IsNullOrEmpty( TsmName ) )
        {
          s += " ";
        }
      }

      if ( this.TsmParent is TsmSubFsm )
      {
        return s + "{" + TsmName + "}";
      }
      return s + TsmName;
    }

    public virtual string FullName()
    {
      return "( " + TotalName() + " )";
    }
  }

  public class TsmPackage : TsmBase
  {
  }

  [Synchronization]
  public class Tsm
  {
    private static StreamWriter mStreamWriter = null;
    private static int          mLogLevel = 10;

    public static void Assert( bool bCondition )
    {
      if ( ! bCondition )
      {
        Console.WriteLine( "ASSERT @ " + System.Environment.StackTrace );
        Tsm.Log( "ASSERT @ " + System.Environment.StackTrace );
      }
    } 

    public static void Log( string s )
    {
      if ( null == mStreamWriter )
      {
        string sFile = FreeLogFile();
        FileStream fs = new FileStream( sFile, FileMode.Create, FileAccess.Write, FileShare.Write );
        mStreamWriter = new StreamWriter( fs );
      }
      mStreamWriter.WriteLine( s );
      mStreamWriter.Flush();

      Console.WriteLine( s );
    }

    public static void SetLogLevel( int iLevel )
    {
      mLogLevel = iLevel;
    }

    public static void Lgl( int iLevel, string s )
    {
      if ( iLevel <= mLogLevel )
      {
        Log( s );
      }
    }

    private static string FreeLogFile()
    {
      string s;
      int ix = 0;
      do
      {
        ix++;
        s = string.Format( "TismLog_{0,4:D4}.txt", ix );
      } while ( File.Exists( s ) );

      return s;
    }

    public static void Close()
    {
      if ( null != mStreamWriter )
      {
        mStreamWriter.Close();
      }
    }

    public static void ReportBox( string s )
    {
      Console.WriteLine( "Report: " + s );
    }
  }

  public class TsmEnvelope
  {
    private TsmTrigger  mTrigger;
    private TsmClass    mTarget;

    public TsmEnvelope( TsmTrigger oTrigger, TsmClass oTarget )
    {
      this.mTrigger = oTrigger;
      this.mTarget  = oTarget;
    }

    public TsmTrigger Trigger
    {
      get { return mTrigger; }
    }

    public TsmClass Target
    {
      get { return mTarget; }
    }

    public string Snap()
    {
      return Trigger.Snap();
    }
  }

  public enum TsmRvState
  {
    IDLE,
    WAIT,
    READY
  }

  public class TsmTrigger
  {
    public int          PortId;
    public int          Command;
    public TsmRvState   RvState = TsmRvState.IDLE;

    public TsmTrigger( int iCmd )
    {
      this.Command = iCmd;
    }

    public void Ready()
    {
      switch ( RvState )
      {
      case TsmRvState.IDLE:
        TsmRecorder.Show( "********** RENDEZVOUS IDLE **********" );
        break;

      case TsmRvState.WAIT:
        RvState = TsmRvState.READY;
        break;

      case TsmRvState.READY:
        TsmRecorder.Show( "********** RENDEZVOUS READY **********" );
        break;
      }
    }

    public string Snap()
    {
      return " " + PortId + "." + Command;
    }
  }
}
