﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace UiCalculator
{
  class View
  {
    private static System.Globalization.CultureInfo mCi = System.Globalization.CultureInfo.InvariantCulture;
    private static System.Globalization.NumberFormatInfo mNfi = mCi.NumberFormat;

    private static string sAll = "";
    private static char   cOper = '\0';
    private static double dPrevious = 0.0;
    private static double dValue = 0.0;

    public static void CancelEntry()
    {
      dValue = 0.0;
      Clear();
      Show();
    }

    private static void Clear()
    {
      sAll = "";
    }

    public static void Convert()
    {
      try
      {
        dValue = double.Parse( sAll, mNfi );
      }
      catch
      {
        dValue = 0;
      }
      Clear();
    }

    public static void Insert( char keyId )
    {
      sAll += keyId;
      Show();
    }

    public static void Inserts( string keyIds )
    {
      sAll += keyIds;
      Show();
    }

    public static void Negate()
    {
      sAll = "-";
      Show();
    }

    public static void Oper( char keyId )
    {
      dPrevious = dValue;
      cOper = keyId;
    }

    public static bool Evaluate()
    {
      Convert();

      switch ( cOper )
      {
      case '+' :
        dValue = dPrevious + dValue;
        break;
      case '-' :
        dValue = dPrevious - dValue;
        break;
      case '*' :
        dValue = dPrevious * dValue;
        break;
      case '/' :
        if ( 0.0 == dValue )
        {
          Show( "Error" );
          return false;
        }
        dValue = dPrevious / dValue;
        break;
      }
      Show( dValue.ToString( mNfi ) );
      return true;
    }

    public static void Show()
    {
      string s = "";

      if ( string.IsNullOrEmpty( sAll ) )
      {
        s += "0";
      }
      else
      {
        s += sAll;
      }
      //s = string.Format( "[{0,12}]", s );
      Show( s );
    }

    public static void Show( string s )
    {
      if ( null != Storage.TextBoxDisplay )
      {
        Storage.TextBoxDisplay.Text = s;
      }
      else
      {
        Console.WriteLine( s );
      }
    }

    public static void Exit()
    {
      Show( "Exit (reached final state)" );
    }


    public static void TsmRun( object oObject )
    {
      TsmIPort_ICalculator Port = (TsmIPort_ICalculator) oObject;

      // MyCode
      for ( ; ; )
      {
        Console.Write( "> " );
        ConsoleKeyInfo cki = Console.ReadKey();
        char [] a = cki.Key.ToString().ToLower().ToCharArray();
        char c = cki.KeyChar; //a[0];
        Console.WriteLine();

        switch ( c )
        {
        case '+':
        case '-':
        case '*':
        case '/':
          Port.Oper( c );
          break;
        case '=':
          Port.Equals();
          break;
        case '.':
          Port.Point();
          break;
        case '0':
          Port.Digit_0();
          break;
        case '1':
        case '2':
        case '3':
        case '4':
        case '5':
        case '6':
        case '7':
        case '8':
        case '9':
          Port.Digit_1_9( c );
          break;
        case 'e':
          Port.CancelEntry();
          break;
        case 'c':
          Port.Cancel();
          break;
        case 'q':
          Port.Off();
          break;

        case '\0':
          break;
        default:
          Console.WriteLine( "Illegal input: " + c );
          break;
        }
      }
    }
  }
}

